﻿unit Publisher;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Subscriber;

type

  { TPublisher }

  TPublisher = class
  private
    FList: TStringList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    function SubscriptionIndex(ASubscriber: TSubscriber; AEvent: string): longint;
    procedure Subscribe(ASubscriber: TSubscriber; AEvent: string); virtual;
    procedure Unsubscribe(ASubscriber: TSubscriber; AEvent: string); virtual;
    procedure Notify(AEvent: string); virtual;
  end;

implementation

{ TPublisher }

function TPublisher.SubscriptionIndex(ASubscriber: TSubscriber; AEvent: string): longint;
{ Возвращает индекс подписки объекта ASubscriber на событие AEvent.
  -1, если не подписан }
var i: longint=0;
begin
  Result:=-1;
  while (i<FList.Count) do
    begin
      if ((FList.Strings[i]=AEvent) and (FList.Objects[i]=ASubscriber)) then
           Result:=i;
      i:=i+1;
    end;
end;

constructor TPublisher.Create;
begin
  FList:=TStringList.Create;
end;

destructor TPublisher.Destroy;
begin
  FreeAndNil(FList);
  inherited Destroy;
end;

procedure TPublisher.Subscribe(ASubscriber: TSubscriber; AEvent: string);
{ Подписывает объект ASubscriber на событие AEvent }
begin
  if SubscriptionIndex(ASubscriber, AEvent) = -1 then
     FList.AddObject(AEvent, ASubscriber);
end;

procedure TPublisher.Unsubscribe(ASubscriber: TSubscriber; AEvent: string);
{ Подписывает объект ASubscriber на событие AEvent }
begin
  if SubscriptionIndex(ASubscriber, AEvent) > -1 then
     FList.Delete(SubscriptionIndex(ASubscriber, AEvent));
end;

procedure TPublisher.Notify(AEvent: string);
{ Уведомляет всех подписчиков на событие AEvent о том, что оно случилось. }
var i: longint;
begin
  for i:=0 to FList.Count-1 do
    if AEvent=FList.Strings[i] then
       (FList.Objects[i] as ISubscriber).Notify(AEvent);
end;

end.

