unit PublisherTestCase; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry,
  Publisher, Subscriber,
  checkMem;

type

  { TPublisherTestCase }

  TPublisherTestCase = class(TTestCase)
  protected
    procedure SetUp; override; 
    procedure TearDown; override;
    P: TPublisher;
  published
    procedure MemoryLeakTest;
    procedure SubscriptionIndexText;
    procedure NotifyTest;
  end; 

  { TTestSubscriber }

  TTestSubscriber = class (TSubscriber)
  private
    FNotifyRecieved: string;
  public
    property NotifyRecieved: string read FNotifyRecieved write FNotifyRecieved;
    procedure Notify(AEvent: string); override;
  end;

implementation

procedure TPublisherTestCase.MemoryLeakTest;
var pub: TPublisher;
    mem: longint;
begin
  mem:=getlost;
  pub:=TPublisher.Create;
  FreeAndNil(pub);
  AssertEquals('Memory leaks detected', 0, getlost - mem);
end;

procedure TPublisherTestCase.SubscriptionIndexText;
var event: string;
    s1, s2, s3: TTestSubscriber;
begin
  s1:=TTestSubscriber.Create(nil);
  s2:=TTestSubscriber.Create(nil);
  s3:=TTestSubscriber.Create(nil);
  p.Subscribe(s1, 'event1');
  p.Subscribe(s2, 'event2');
  p.Subscribe(s3, 'event3');
  p.Subscribe(s2, 'event4');
  AssertEquals('SubscriptionIndex not found the subscription', 2,
        p.SubscriptionIndex(s3, 'event3'));
  FreeAndNil(s1);
  FreeAndNil(s2);
  FreeAndNil(s3);
end;

procedure TPublisherTestCase.NotifyTest;
var event: string;
    s1, s2, s3: TTestSubscriber;
begin
  s1:=TTestSubscriber.Create(nil);
  s2:=TTestSubscriber.Create(nil);
  s3:=TTestSubscriber.Create(nil);
  p.Subscribe(s1, 'event1');
  p.Subscribe(s2, 'event2');
  p.Subscribe(s3, 'event3');
  p.Subscribe(s2, 'event4');
  p.Notify('event2');
  AssertEquals('Subscriber was not notified', 'event2', s2.NotifyRecieved);
  FreeAndNil(s1);
  FreeAndNil(s2);
  FreeAndNil(s3);
end;

procedure TPublisherTestCase.SetUp; 
begin
  P:=TPublisher.Create;
end; 

procedure TPublisherTestCase.TearDown; 
begin
  FreeAndNil(P);
end; 

{ TTestSubscriber }

procedure TTestSubscriber.Notify(AEvent: string);
begin
  inherited Notify(AEvent);
  FNotifyRecieved:=AEvent;
end;

initialization

  RegisterTest(TPublisherTestCase); 
end.

